@@ -32,10 +32,15 @@ def setup_resources(feature):
     assert predictions['code'] == HTTP_OK
     world.init_predictions_count = predictions['meta']['total_count']
 
+    evaluations = world.api.list_evaluations()
+    assert evaluations['code'] == HTTP_OK
+    world.init_evaluations_count = evaluations['meta']['total_count']
+
     world.sources = []
     world.datasets = []
     world.models = []
     world.predictions = []
+    world.evaluations = []
 
 @after.each_feature
 def cleanup_resources(feature):
@@ -59,6 +64,10 @@ def cleanup_resources(feature):
         world.api.delete_prediction(id)
     world.predictions = []
 
+    for id in world.evaluations:
+        world.api.delete_evaluation(id)
+    world.evaluations = []
+
     sources = world.api.list_sources()
     assert sources['code'] == HTTP_OK
     world.final_sources_count = sources['meta']['total_count']
@@ -75,7 +84,12 @@ def cleanup_resources(feature):
     assert predictions['code'] == HTTP_OK
     world.final_predictions_count = predictions['meta']['total_count']
 
+    evaluations = world.api.list_evaluations()
+    assert evaluations['code'] == HTTP_OK
+    world.final_evaluations_count = evaluations['meta']['total_count']
+
     assert world.final_sources_count == world.init_sources_count
     assert world.final_datasets_count == world.init_datasets_count
     assert world.final_models_count == world.init_models_count
     assert world.final_predictions_count == world.init_predictions_count
+    assert world.final_evaluations_count == world.init_evaluations_count
